@@ -22,14 +22,15 @@
 #include "config.h"
 
 #include "seahorse-service.h"
-#include "seahorse-key.h"
+#include "seahorse-object.h"
 #include "seahorse-gconf.h"
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
 
-#include "pgp/seahorse-pgp-dialogs.h"
-#include "pgp/seahorse-pgp-key.h"
-#include "pgp/seahorse-pgp-operation.h"
+#include "pgp/seahorse-gpgme.h"
+#include "pgp/seahorse-gpgme-dialogs.h"
+#include "pgp/seahorse-gpgme-key.h"
+#include "pgp/seahorse-gpgme-operation.h"
 
 #include <string.h>
 
@@ -61,7 +62,7 @@ seahorse_service_crypto_new ()
 
 /* Returns result in result. Frees pop and cryptdata */
 static gboolean
-process_crypto_result (SeahorsePGPOperation *pop, gpgme_error_t gstarterr, 
+process_crypto_result (SeahorseGpgmeOperation *pop, gpgme_error_t gstarterr, 
                        gpgme_data_t cryptdata, gchar **result, GError **error)
 {
     size_t len;
@@ -113,23 +114,35 @@ keylist_to_keys (GList *keys)
 	recips = g_new0 (gpgme_key_t, g_list_length (keys) + 1);
 
 	for (i = 0; keys != NULL; keys = g_list_next (keys), i++) {
-		g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (keys->data), recips);
-		recips[i] = SEAHORSE_PGP_KEY (keys->data)->pubkey;
-		gpgmex_key_ref (recips[i]);
+		g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (keys->data), recips);
+		recips[i] = seahorse_gpgme_key_get_public (keys->data);
+		gpgme_key_ref (recips[i]);
 	}
 
 	return recips;
 }
 
+static void
+free_keys (gpgme_key_t* keys)
+{
+	gpgme_key_t* k = keys;
+	if (!keys)
+		return;
+	while (*k)
+		gpgme_key_unref (*(k++));
+	g_free (keys);
+}
+
+
 static void
 notify_signatures (const gchar* data, gpgme_verify_result_t status)
 {
 	const gchar *icon = NULL;
+	SeahorseObject *object;
 	gchar *title, *body;
 	gboolean sig = FALSE;
 	GSList *rawids;
 	GList *keys;
-	SeahorseKey *skey;
 
 	/* Discover the key in question */
 	rawids = g_slist_append (NULL, status->signatures->fpr);
@@ -137,7 +150,7 @@ notify_signatures (const gchar* data, gpgme_verify_result_t status)
 	g_slist_free (rawids);
 
 	g_return_if_fail (keys != NULL);
-	skey = SEAHORSE_KEY (keys->data);
+	object = SEAHORSE_OBJECT (keys->data);
 	g_list_free (keys);
 
 	/* Figure out what to display */
@@ -184,14 +197,14 @@ notify_signatures (const gchar* data, gpgme_verify_result_t status)
 		return;
 	default:
 		if (!GPG_IS_OK (status->signatures->status))
-			seahorse_pgp_handle_gpgme_error (status->signatures->status, 
-			                            _("Couldn't verify signature."));
+			seahorse_gpgme_handle_error (status->signatures->status, 
+			                             _("Couldn't verify signature."));
 		return;
 	};
 
 	if (sig) {
 		gchar *date = seahorse_util_get_display_date_string (status->signatures->timestamp);
-		gchar *id = seahorse_context_id_to_dbus (SCTX_APP (), seahorse_key_get_keyid (skey), 0);
+		gchar *id = seahorse_context_id_to_dbus (SCTX_APP (), seahorse_object_get_id (object));
 		body = g_markup_printf_escaped (body, id, date);
 		g_free (date);
 		g_free (id);
@@ -223,7 +236,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
                                       char **crypttext, GError **error)
 {
     GList *recipkeys = NULL;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     SeahorseObject *signkey = NULL;
     SeahorseObject *skey;
     gpgme_key_t *recips;
@@ -238,15 +251,15 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     
     /* The signer */
     if (signer && signer[0]) {
-        signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
+        signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer);
         if (!signkey) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                          _("Invalid or unrecognized signer: %s"), signer);
             return FALSE;
         }
         
-        if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-            !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+        if (!SEAHORSE_IS_GPGME_KEY (signkey) || 
+            !(seahorse_object_get_flags (signkey) & SEAHORSE_FLAG_CAN_SIGN)) {
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not valid for signing: %s"), signer);
             return FALSE;
@@ -256,7 +269,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     /* The recipients */
     for( ; recipients[0]; recipients++)
     {
-        skey = seahorse_context_object_from_dbus (SCTX_APP (), recipients[0], NULL);
+        skey = seahorse_context_object_from_dbus (SCTX_APP (), recipients[0]);
         if (!skey) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
@@ -264,8 +277,8 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
             return FALSE;
         }
         
-        if (!SEAHORSE_IS_PGP_KEY (skey) ||
-            !(seahorse_object_get_flags (skey) & SKEY_FLAG_CAN_ENCRYPT)) {
+        if (!SEAHORSE_IS_GPGME_KEY (skey) ||
+            !(seahorse_object_get_flags (skey) & SEAHORSE_FLAG_CAN_ENCRYPT)) {
             g_list_free (recipkeys);
             g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                          _("Key is not a valid recipient for encryption: %s"), recipients[0]);
@@ -281,7 +294,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
         return FALSE;
     }
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data form text */
     gerr = gpgme_data_new_from_mem (&plain, cleartext, strlen (cleartext), FALSE);
@@ -306,7 +319,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
     
     /* Do the encryption */
     if (signkey) {
-        gpgme_signers_add (pop->gctx, SEAHORSE_PGP_KEY (signkey)->seckey);
+        gpgme_signers_add (pop->gctx, seahorse_gpgme_key_get_private (SEAHORSE_GPGME_KEY (signkey)));
         gerr = gpgme_op_encrypt_sign_start (pop->gctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, 
                                             plain, cipher);
     } else {
@@ -314,7 +327,7 @@ seahorse_service_crypto_encrypt_text (SeahorseServiceCrypto *crypto,
                                        plain, cipher);
     }
     
-    gpgmex_free_keys (recips);
+    free_keys (recips);
 
     /* Frees cipher */
     ret = process_crypto_result (pop, gerr, cipher, crypttext, error);
@@ -331,7 +344,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
 {
     SeahorseObject *signkey = NULL;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     
@@ -345,21 +358,21 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("No signer specified"));
     
-    signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer, NULL);
+    signkey = seahorse_context_object_from_dbus (SCTX_APP (), signer);
     if (!signkey) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
                      _("Invalid or unrecognized signer: %s"), signer);
         return FALSE;
     }
         
-    if (!SEAHORSE_IS_PGP_KEY (signkey) || 
-        !(seahorse_object_get_flags (signkey) & SKEY_FLAG_CAN_SIGN)) {
+    if (!SEAHORSE_IS_GPGME_KEY (signkey) || 
+        !(seahorse_object_get_flags (signkey) & SEAHORSE_FLAG_CAN_SIGN)) {
         g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID,
                      _("Key is not valid for signing: %s"), signer);
         return FALSE;
     }
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&plain, cleartext, strlen (cleartext), FALSE);
@@ -372,7 +385,7 @@ seahorse_service_crypto_sign_text (SeahorseServiceCrypto *crypto, const char *si
     gpgme_set_armor (pop->gctx, TRUE);
 
     /* Do the signage */
-    gpgme_signers_add (pop->gctx, SEAHORSE_PGP_KEY (signkey)->seckey);
+    gpgme_signers_add (pop->gctx, seahorse_gpgme_key_get_private (SEAHORSE_GPGME_KEY (signkey)));
     gerr = gpgme_op_sign_start (pop->gctx, plain, cipher, GPGME_SIG_MODE_CLEAR);
 
     /* Frees cipher */
@@ -391,7 +404,7 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
 {
     gpgme_verify_result_t status;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     GQuark keyid;
@@ -407,7 +420,7 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
      * then all this logic will need to change. 
      */
     
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&cipher, crypttext, strlen (crypttext), FALSE);
@@ -435,8 +448,8 @@ seahorse_service_crypto_decrypt_text (SeahorseServiceCrypto *crypto,
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
                 status->signatures->summary & GPGME_SIGSUM_VALID ||
                 status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
-                keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
+                keyid = seahorse_pgp_key_calc_cannonical_id (status->signatures->fpr);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid);
             }
         }
     }
@@ -454,7 +467,7 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
 {
     gpgme_verify_result_t status;
     gpgme_error_t gerr;
-    SeahorsePGPOperation *pop; 
+    SeahorseGpgmeOperation *pop; 
     gpgme_data_t plain, cipher;
     gboolean ret = TRUE;
     GQuark keyid;
@@ -470,7 +483,7 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
      * then all this logic will need to change. 
      */
 
-    pop = seahorse_pgp_operation_new (NULL);
+    pop = seahorse_gpgme_operation_new (NULL);
     
     /* new data from text */
     gerr = gpgme_data_new_from_mem (&cipher, crypttext, strlen (crypttext), FALSE);
@@ -498,8 +511,8 @@ seahorse_service_crypto_verify_text (SeahorseServiceCrypto *crypto,
             if (status->signatures->summary & GPGME_SIGSUM_GREEN ||
                 status->signatures->summary & GPGME_SIGSUM_VALID ||
                 status->signatures->summary & GPGME_SIGSUM_KEY_MISSING) {
-                keyid = seahorse_pgp_key_get_cannonical_id (status->signatures->fpr);
-                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid, 0);
+                keyid = seahorse_pgp_key_calc_cannonical_id (status->signatures->fpr);
+                *signer = seahorse_context_id_to_dbus (SCTX_APP (), keyid);
             }
         }
     }
