@@ -23,9 +23,10 @@
 
 #include <glib/gi18n.h>
 
-#include "seahorse-context.h"
 #include "seahorse-unknown-source.h"
-#include "seahorse-unknown-key.h"
+
+#include "seahorse-context.h"
+#include "seahorse-unknown.h"
 
 #include "common/seahorse-registry.h"
 
@@ -36,7 +37,10 @@ enum {
     PROP_LOCATION
 };
 
-G_DEFINE_TYPE (SeahorseUnknownSource, seahorse_unknown_source, SEAHORSE_TYPE_SOURCE);
+static void seahorse_source_iface (SeahorseSourceIface *iface);
+
+G_DEFINE_TYPE_EXTENDED (SeahorseUnknownSource, seahorse_unknown_source, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_SOURCE, seahorse_source_iface));
 
 /* -----------------------------------------------------------------------------
  * INTERNAL
@@ -53,43 +57,9 @@ search_done (SeahorseOperation *op, SeahorseObject *sobj)
  */
 
 static SeahorseOperation*
-seahorse_unknown_source_load (SeahorseSource *src, GQuark id)
-{
-    SeahorseUnknownSource *usrc;
-    
-    g_assert (SEAHORSE_IS_UNKNOWN_SOURCE (src));
-    usrc = SEAHORSE_UNKNOWN_SOURCE (src);
-    
-    if (id) {
-        g_return_val_if_fail (id, NULL);
-        seahorse_unknown_source_add_object (usrc, id, NULL);
-    }
-    
-    return seahorse_operation_new_complete (NULL);
-}
-
-static SeahorseOperation* 
-seahorse_unknown_source_import (SeahorseSource *sksrc, GInputStream *input)
-{
-    g_return_val_if_reached (NULL);
-    return NULL;
-}
-
-static SeahorseOperation* 
-seahorse_unknown_source_export_raw (SeahorseSource *sksrc, GSList *ids, 
-                                    GOutputStream *output)
-{
-    g_return_val_if_reached (NULL);
-    return NULL;
-}
-
-static gboolean            
-seahorse_unknown_source_remove (SeahorseSource *sksrc, SeahorseObject *sobj,
-                                guint name, GError **error)
-{
-    g_return_val_if_fail (sksrc == seahorse_object_get_source (sobj), FALSE);
-    seahorse_context_remove_object (SCTX_APP (), sobj);
-    return TRUE;
+seahorse_unknown_source_load (SeahorseSource *src)
+{ 
+	return seahorse_operation_new_complete (NULL);
 }
 
 static void 
@@ -119,7 +89,7 @@ seahorse_unknown_source_get_property (GObject *object, guint prop_id, GValue *va
         g_value_set_string (value, _("Unavailable Key"));
         break;
     case PROP_LOCATION:
-        g_value_set_uint (value, SEAHORSE_LOCATION_MISSING);
+        g_value_set_enum (value, SEAHORSE_LOCATION_MISSING);
         break;
     }
 }
@@ -133,32 +103,24 @@ seahorse_unknown_source_init (SeahorseUnknownSource *ssrc)
 static void
 seahorse_unknown_source_class_init (SeahorseUnknownSourceClass *klass)
 {
-    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-    SeahorseSourceClass *parent_class = SEAHORSE_SOURCE_CLASS (klass);
+	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    
-    seahorse_unknown_source_parent_class = g_type_class_peek_parent (klass);
+	seahorse_unknown_source_parent_class = g_type_class_peek_parent (klass);
     
-    gobject_class->set_property = seahorse_unknown_source_set_property;
-    gobject_class->get_property = seahorse_unknown_source_get_property;
+	gobject_class->set_property = seahorse_unknown_source_set_property;
+	gobject_class->get_property = seahorse_unknown_source_get_property;
     
-    parent_class->load = seahorse_unknown_source_load;
-    parent_class->import = seahorse_unknown_source_import;
-    parent_class->export_raw = seahorse_unknown_source_export_raw;
-    parent_class->remove = seahorse_unknown_source_remove;
- 
-    g_object_class_install_property (gobject_class, PROP_KEY_TYPE,
-        g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
-                           0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_override_property (gobject_class, PROP_KEY_TYPE, "key-type");
+	g_object_class_override_property (gobject_class, PROP_KEY_DESC, "key-desc");
+	g_object_class_override_property (gobject_class, PROP_LOCATION, "location");
+    
+	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_UNKNOWN_SOURCE, "source", NULL);
+}
 
-    g_object_class_install_property (gobject_class, PROP_KEY_DESC,
-        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
-                             NULL, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_LOCATION,
-        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseLocation", 
-                           0, G_MAXUINT, SEAHORSE_LOCATION_MISSING, G_PARAM_READABLE));    
-    
-	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_UNKNOWN_SOURCE, "key-source", NULL);
+static void 
+seahorse_source_iface (SeahorseSourceIface *iface)
+{
+	iface->load = seahorse_unknown_source_load;
 }
 
 /* -----------------------------------------------------------------------------
@@ -181,12 +143,12 @@ seahorse_unknown_source_add_object (SeahorseUnknownSource *usrc, GQuark id,
 
     sobj = seahorse_context_get_object (SCTX_APP (), SEAHORSE_SOURCE (usrc), id);
     if (!sobj) {
-        sobj = SEAHORSE_OBJECT (seahorse_unknown_key_new (usrc, id));
+        sobj = SEAHORSE_OBJECT (seahorse_unknown_new (usrc, id, NULL));
         seahorse_context_add_object (SCTX_APP (), sobj);
     }
     
     if (search) {
-        sobj->_location = SEAHORSE_LOCATION_SEARCHING;
+        g_object_set (sobj, "location", SEAHORSE_LOCATION_SEARCHING, NULL);
         seahorse_operation_watch (search, (SeahorseDoneFunc) search_done, sobj, NULL, NULL);
     }
     
